@@ -4,6 +4,7 @@
 #' @include translate-sql-paste.R
 #' @include translate-sql-string.R
 #' @include translate-sql-quantile.R
+#' @include translate-sql-cut.R
 #' @include escape.R
 #' @include sql.R
 #' @include utils.R
@@ -22,13 +23,27 @@ sql_translation.DBIConnection <- function(con) {
 #' @rdname sql_variant
 #' @format NULL
 base_scalar <- sql_translator(
-  `+`    = sql_infix("+"),
+  `+`    = function(x, y = NULL) {
+    x <- escape_infix_expr(enexpr(x), x, escape_unary_minus = TRUE)
+    if (is.null(y)) {
+      if (is.numeric(x)) {
+        x
+      } else {
+        sql_expr(!!x)
+      }
+    } else {
+      y <- escape_infix_expr(enexpr(y), y)
+
+      sql_expr(!!x + !!y)
+    }
+  },
   `*`    = sql_infix("*"),
   `/`    = sql_infix("/"),
   `%/%`  = sql_not_supported("%/%"),
   `%%`   = sql_infix("%"),
   `^`    = sql_prefix("POWER", 2),
   `-`    = function(x, y = NULL) {
+    x <- escape_infix_expr(enexpr(x), x, escape_unary_minus = TRUE)
     if (is.null(y)) {
       if (is.numeric(x)) {
         -x
@@ -36,6 +51,8 @@ base_scalar <- sql_translator(
         sql_expr(-!!x)
       }
     } else {
+      y <- escape_infix_expr(enexpr(y), y)
+
       sql_expr(!!x - !!y)
     }
   },
@@ -48,7 +65,7 @@ base_scalar <- sql_translator(
     } else if (is.numeric(i)) {
       build_sql(x, "[", as.integer(i), "]")
     } else {
-      stop("Can only index with strings and numbers", call. = FALSE)
+      cli_abort("Can only index with strings and numbers")
     }
 
   },
@@ -134,9 +151,15 @@ base_scalar <- sql_translator(
     sql_expr(ROUND(!!x, !!as.integer(digits)))
   },
 
-  `if` = sql_if,
-  if_else = function(condition, true, false) sql_if(condition, true, false),
-  ifelse = function(test, yes, no) sql_if(test, yes, no),
+  `if` = function(cond, if_true, if_false = NULL) {
+    sql_if(enquo(cond), enquo(if_true), enquo(if_false))
+  },
+  if_else = function(condition, true, false, missing = NULL) {
+    sql_if(
+      enquo(condition), enquo(true), enquo(false), enquo(missing)
+    )
+  },
+  ifelse = function(test, yes, no) sql_if(enquo(test), enquo(yes), enquo(no)),
 
   switch = function(x, ...) sql_switch(x, ...),
   case_when = function(...) sql_case_when(...),
@@ -221,6 +244,7 @@ base_scalar <- sql_translator(
   paste0 = sql_paste(""),
   substr = sql_substr("SUBSTR"),
   substring = sql_substr("SUBSTR"),
+  cut = sql_cut,
 
   # stringr functions
   str_length = sql_prefix("LENGTH", 1),
@@ -381,7 +405,13 @@ base_win <- sql_translator(
 
   # Counts
   n     = function() {
-    win_over(sql("COUNT(*)"), win_current_group())
+    frame <- win_current_frame()
+    win_over(
+      sql("COUNT(*)"),
+      partition = win_current_group(),
+      order = if (!is.null(frame)) win_current_order(),
+      frame = frame
+    )
   },
   n_distinct = function(x) {
     win_over(build_sql("COUNT(DISTINCT ", x, ")"), win_current_group())
@@ -439,3 +469,10 @@ base_no_win <- sql_translator(
   str_flatten  = win_absent("STR_FLATTEN"),
   count        = win_absent("COUNT")
 )
+
+#' @export
+sql_random.DBIConnection <- function(con) {
+  sql_expr(RANDOM())
+}
+
+utils::globalVariables("RANDOM")
