@@ -1,7 +1,7 @@
 #' Backend: SQL server
 #'
 #' @description
-#' See `vignette("translate-function")` and `vignette("translate-verb")` for
+#' See `vignette("translation-function")` and `vignette("translation-verb")` for
 #' details of overall translation technology. Key differences for this backend
 #' are:
 #'
@@ -55,6 +55,9 @@
 #' lf %>% transmute(x = ifelse(c > d, "c", "d"))
 NULL
 
+#' @include verb-copy-to.R
+NULL
+
 #' @export
 #' @rdname simulate_dbi
 simulate_mssql <- function(version = "15.0") {
@@ -71,21 +74,132 @@ simulate_mssql <- function(version = "15.0") {
 #' @export
 `sql_query_select.Microsoft SQL Server` <- function(con, select, from, where = NULL,
                                              group_by = NULL, having = NULL,
+                                             window = NULL,
                                              order_by = NULL,
                                              limit = NULL,
                                              distinct = FALSE,
                                              ...,
-                                             subquery = FALSE) {
+                                             subquery = FALSE,
+                                             lvl = 0) {
   sql_select_clauses(con,
     select    = sql_clause_select(con, select, distinct, top = limit),
-    from      = sql_clause_from(con, from),
-    where     = sql_clause_where(con, where),
-    group_by  = sql_clause_group_by(con, group_by),
-    having    = sql_clause_having(con, having),
-    order_by  = sql_clause_order_by(con, order_by, subquery, limit)
+    from      = sql_clause_from(from),
+    where     = sql_clause_where(where),
+    group_by  = sql_clause_group_by(group_by),
+    having    = sql_clause_having(having),
+    window    = sql_clause_window(window),
+    order_by  = sql_clause_order_by(order_by, subquery, limit),
+    lvl       = lvl
   )
 }
 
+#' @export
+`sql_query_insert.Microsoft SQL Server` <- function(con, x_name, y, by, ...,
+                                                    conflict = c("error", "ignore"),
+                                                    returning_cols = NULL,
+                                                    method = NULL) {
+  method <- method %||% "where_not_exists"
+  arg_match(method, "where_not_exists", error_arg = "method")
+  # https://stackoverflow.com/questions/25969/insert-into-values-select-from
+  conflict <- rows_check_conflict(conflict)
+
+  parts <- rows_insert_prep(con, x_name, y, by, lvl = 0)
+
+  clauses <- list2(
+    parts$insert_clause,
+    sql_returning_cols(con, returning_cols, "INSERTED"),
+    sql_clause_select(con, sql("*")),
+    sql_clause_from(parts$from),
+    !!!parts$conflict_clauses
+  )
+
+  sql_format_clauses(clauses, lvl = 0, con)
+}
+
+#' @export
+`sql_query_append.Microsoft SQL Server` <- function(con, x_name, y, ...,
+                                                    returning_cols = NULL) {
+  parts <- rows_prep(con, x_name, y, by = list(), lvl = 0)
+  insert_cols <- escape(ident(colnames(y)), collapse = ", ", parens = TRUE, con = con)
+
+  clauses <- list2(
+    sql_clause_insert(con, insert_cols, x_name),
+    sql_returning_cols(con, returning_cols, "INSERTED"),
+    sql_clause_select(con, sql("*")),
+    sql_clause_from(parts$from)
+  )
+
+  sql_format_clauses(clauses, lvl = 0, con)
+}
+
+#' @export
+`sql_query_update_from.Microsoft SQL Server` <- function(con, x_name, y, by,
+                                                         update_values, ...,
+                                                         returning_cols = NULL) {
+  # https://stackoverflow.com/a/2334741/946850
+  parts <- rows_prep(con, x_name, y, by, lvl = 0)
+  update_cols <- sql_escape_ident(con, names(update_values))
+
+  clauses <- list(
+    sql_clause_update(x_name),
+    sql_clause_set(update_cols, update_values),
+    sql_returning_cols(con, returning_cols, "INSERTED"),
+    sql_clause_from(x_name),
+    sql_clause("INNER JOIN", parts$from),
+    sql_clause_on(parts$where, lvl = 1)
+  )
+  sql_format_clauses(clauses, lvl = 0, con)
+}
+
+#' @export
+`sql_query_upsert.Microsoft SQL Server` <- function(con,
+                                                    x_name,
+                                                    y,
+                                                    by,
+                                                    update_cols,
+                                                    ...,
+                                                    returning_cols = NULL,
+                                                    method = NULL) {
+  method <- method %||% "merge"
+  arg_match(method, "merge", error_arg = "method")
+
+  parts <- rows_prep(con, x_name, y, by, lvl = 0)
+
+  update_cols_esc <- sql(sql_escape_ident(con, update_cols))
+  update_values <- sql_table_prefix(con, update_cols, ident("...y"))
+  update_clause <- sql(paste0(update_cols_esc, " = ", update_values))
+
+  insert_cols <- c(by, update_cols)
+  insert_cols_esc <- sql(sql_escape_ident(con, insert_cols))
+  insert_cols_qual <- sql_table_prefix(con, insert_cols, ident("...y"))
+
+  clauses <- list(
+    sql_clause("MERGE INTO", x_name),
+    sql_clause("USING", parts$from),
+    sql_clause_on(parts$where, lvl = 1),
+    sql("WHEN MATCHED THEN"),
+    sql_clause("UPDATE SET", update_clause, lvl = 1),
+    sql("WHEN NOT MATCHED THEN"),
+    sql_clause_insert(con, insert_cols_esc, lvl = 1),
+    sql_clause("VALUES", insert_cols_qual, parens = TRUE, lvl = 1),
+    sql_returning_cols(con, returning_cols, "INSERTED"),
+    sql(";")
+  )
+  sql_format_clauses(clauses, lvl = 0, con)
+}
+
+#' @export
+`sql_query_delete.Microsoft SQL Server` <- function(con, x_name, y, by, ..., returning_cols = NULL) {
+  parts <- rows_prep(con, x_name, y, by, lvl = 0)
+
+  clauses <- list2(
+    sql_clause("DELETE FROM", x_name),
+    sql_returning_cols(con, returning_cols, table = "DELETED"),
+    !!!sql_clause_where_exists(parts$from, parts$where, not = FALSE)
+  )
+  sql_format_clauses(clauses, lvl = 0, con)
+}
+
 #' @export
 `sql_translation.Microsoft SQL Server` <- function(con) {
   mssql_scalar <-
@@ -112,12 +226,23 @@ simulate_mssql <- function(version = "15.0") {
       `|`            = mssql_infix_boolean("|", "%OR%"),
       `||`           = mssql_infix_boolean("|", "%OR%"),
 
+      `[` = function(x, i) {
+        i <- with_mssql_bool(i)
+        build_sql("CASE WHEN (", i, ") THEN (", x, ") END")
+      },
+
       bitwShiftL     = sql_not_supported("bitwShiftL"),
       bitwShiftR     = sql_not_supported("bitwShiftR"),
 
-      `if`           = mssql_sql_if,
-      if_else        = function(condition, true, false) mssql_sql_if(condition, true, false),
-      ifelse         = function(test, yes, no) mssql_sql_if(test, yes, no),
+      `if`           = function(condition, true, false = NULL, missing = NULL) {
+        mssql_sql_if(enquo(condition), enquo(true), enquo(false), enquo(missing))
+      },
+      if_else        = function(condition, true, false, missing = NULL) {
+        mssql_sql_if(enquo(condition), enquo(true), enquo(false), enquo(missing))
+      },
+      ifelse         = function(test, yes, no) {
+        mssql_sql_if(enquo(test), enquo(yes), enquo(no))
+      },
       case_when      = mssql_case_when,
 
       as.logical    = sql_cast("BIT"),
@@ -150,7 +275,8 @@ simulate_mssql <- function(version = "15.0") {
       str_c = sql_paste_infix("", "+", function(x) sql_expr(cast(!!x %as% text))),
       # no built in function: https://stackoverflow.com/questions/230138
       str_to_title = sql_not_supported("str_to_title()"),
-      str_sub = sql_str_sub("SUBSTRING", "LEN"),
+      # https://docs.microsoft.com/en-us/sql/t-sql/functions/substring-transact-sql?view=sql-server-ver15
+      str_sub = sql_str_sub("SUBSTRING", "LEN", optional_length = FALSE),
 
       # lubridate ---------------------------------------------------------------
       # https://en.wikibooks.org/wiki/SQL_Dialects_Reference/Functions_and_expressions/Date_and_time_functions
@@ -179,13 +305,14 @@ simulate_mssql <- function(version = "15.0") {
           if (!abbr) {
             sql_expr(DATENAME(MONTH, !!x))
           } else {
-            stop("`abbr` is not supported in SQL Server translation", call. = FALSE)          }
+            cli_abort("{.arg abbr} is not supported in SQL Server translation")
+          }
         }
       },
 
       quarter = function(x, with_year = FALSE, fiscal_start = 1) {
         if (fiscal_start != 1) {
-          stop("`fiscal_start` is not supported in SQL Server translation. Must be 1.", call. = FALSE)
+          cli_abort("{.arg fiscal_start} is not supported in SQL Server translation. Must be 1.")
         }
 
         if (with_year) {
@@ -204,9 +331,18 @@ simulate_mssql <- function(version = "15.0") {
       as.POSIXct = sql_try_cast("TIMESTAMP"),
       as.numeric = sql_try_cast("FLOAT"),
       as.double = sql_try_cast("FLOAT"),
-      as.integer = sql_try_cast("NUMERIC"),
-      # in MSSQL, NUMERIC converts to integer
-      as.integer64 = sql_try_cast("BIGINT"),
+
+      # In SQL server, CAST (even with TRY) of INTEGER and BIGINT appears
+      # fill entire columns with NULL if parsing single value fails:
+      # https://gist.github.com/DavidPatShuiFong/7b47a9804a497b605e477f1bf6c38b37
+      # So we parse to NUMERIC (which doesn't have this problem), then to the
+      # target type
+      as.integer = function(x) {
+        sql_expr(try_cast(try_cast(!!x %as% NUMERIC) %as% INT))
+      },
+      as.integer64 = function(x) {
+        sql_expr(try_cast(try_cast(!!x %as% NUMERIC(38L, 0L)) %as% BIGINT))
+      },
       as.character = sql_try_cast("VARCHAR(MAX)"),
       as_date = sql_try_cast("DATE"),
       as_datetime = sql_try_cast("DATETIME2")
@@ -221,7 +357,11 @@ simulate_mssql <- function(version = "15.0") {
                       # MSSQL does not have function for: cor and cov
       cor           = sql_not_supported("cor()"),
       cov           = sql_not_supported("cov()"),
-      str_flatten = function(x, collapse = "") sql_expr(string_agg(!!x, !!collapse))
+      str_flatten = function(x, collapse = "") sql_expr(string_agg(!!x, !!collapse)),
+
+      # percentile_cont needs `OVER()` in mssql
+      # https://docs.microsoft.com/en-us/sql/t-sql/functions/percentile-cont-transact-sql?view=sql-server-ver15
+      quantile = sql_quantile("PERCENTILE_CONT", "ordered", window = TRUE)
 
     ),
     sql_translator(.parent = base_odbc_win,
@@ -245,7 +385,7 @@ mssql_version <- function(con) {
   if (inherits(con, "TestConnection")) {
     attr(con, "version")
   } else {
-    numeric_version(DBI::dbGetInfo(con)$db.version)
+    numeric_version(DBI::dbGetInfo(con)$db.version) # nocov
   }
 }
 
@@ -259,7 +399,7 @@ mssql_version <- function(con) {
 #' @export
 `sql_table_analyze.Microsoft SQL Server` <- function(con, table, ...) {
   # https://docs.microsoft.com/en-us/sql/t-sql/statements/update-statistics-transact-sql
-  build_sql("UPDATE STATISTICS ", as.sql(table), con = con)
+  build_sql("UPDATE STATISTICS ", as.sql(table, con = con), con = con)
 }
 
 # SQL server does not use CREATE TEMPORARY TABLE and instead prefixes
@@ -283,12 +423,28 @@ mssql_version <- function(con) {
 
   # https://stackoverflow.com/q/16683758/946850
   build_sql(
-    "SELECT * INTO ", as.sql(name), " ",
-    "FROM (", sql, ") AS temp",
+    "SELECT * INTO ", as.sql(name, con), " ",
+    "FROM (\n  ", sql, "\n) AS temp",
     con = con
   )
 }
 
+#' @export
+`sql_values_subquery.Microsoft SQL Server` <- sql_values_subquery_column_alias
+
+#' @export
+`sql_random.Microsoft SQL Server` <- function(con) {
+  sql_expr(RAND())
+}
+
+#' @export
+`sql_returning_cols.Microsoft SQL Server` <- function(con, cols, table, ...) {
+  stopifnot(table %in% c("DELETED", "INSERTED"))
+  returning_cols <- sql_named_cols(con, cols, table = ident(table))
+
+  sql_clause("OUTPUT", returning_cols)
+}
+
 # Bit vs boolean ----------------------------------------------------------
 
 mssql_needs_bit <- function() {
@@ -336,14 +492,19 @@ mssql_infix_boolean <- function(if_bit, if_bool) {
   }
 }
 
-mssql_sql_if <- function(cond, if_true, if_false = NULL) {
-  cond <- with_mssql_bool(cond)
-  sql_expr(IIF(!!cond, !!if_true, !!if_false))
+mssql_sql_if <- function(cond, if_true, if_false = NULL, missing = NULL) {
+  cond_sql <- with_mssql_bool(eval_tidy(cond))
+  if (is_null(missing) || quo_is_null(missing)) {
+    if_true_sql <- build_sql(eval_tidy(if_true))
+    if_false_sql <- build_sql(eval_tidy(if_false))
+    sql_expr(IIF(!!cond_sql, !!if_true_sql, !!if_false_sql))
+  } else {
+    sql_if(cond, if_true, if_false, missing)
+  }
 }
 
 mssql_case_when <- function(...) {
-  local_context(list(clause = ""))
-  sql_case_when(...)
+  with_mssql_bool(sql_case_when(...))
 }
 
 #' @export
@@ -357,4 +518,4 @@ mssql_case_when <- function(...) {
   y
 }
 
-globalVariables(c("BIT", "CAST", "%AS%", "%is%", "convert", "DATE", "DATENAME", "DATEPART", "IIF", "NOT", "SUBSTRING", "LTRIM", "RTRIM", "CHARINDEX", "SYSDATETIME", "SECOND", "MINUTE", "HOUR", "DAY", "DAYOFWEEK", "DAYOFYEAR", "MONTH", "QUARTER", "YEAR"))
+globalVariables(c("BIT", "CAST", "%AS%", "%is%", "convert", "DATE", "DATENAME", "DATEPART", "IIF", "NOT", "SUBSTRING", "LTRIM", "RTRIM", "CHARINDEX", "SYSDATETIME", "SECOND", "MINUTE", "HOUR", "DAY", "DAYOFWEEK", "DAYOFYEAR", "MONTH", "QUARTER", "YEAR", "BIGINT", "INT"))
