@@ -1,7 +1,7 @@
 #' Backend: PostgreSQL
 #'
 #' @description
-#' See `vignette("translate-function")` and `vignette("translate-verb")` for
+#' See `vignette("translation-function")` and `vignette("translation-verb")` for
 #' details of overall translation technology. Key differences for this backend
 #' are:
 #'
@@ -19,7 +19,10 @@
 #'
 #' lf <- lazy_frame(a = TRUE, b = 1, c = 2, d = "z", con = simulate_postgres())
 #' lf %>% summarise(x = sd(b, na.rm = TRUE))
-#' lf %>% summarise(y = cor(b, c), y = cov(b, c))
+#' lf %>% summarise(y = cor(b, c), z = cov(b, c))
+NULL
+
+#' @include verb-copy-to.R
 NULL
 
 #' @export
@@ -38,7 +41,7 @@ db_connection_describe.PqConnection <- function(con) {
   info <- dbGetInfo(con)
   host <- if (info$host == "") "localhost" else info$host
 
-  paste0("postgres ", info$serverVersion, " [", info$user, "@",
+  paste0("postgres ", info$serverVersion, " [", info$username, "@",
     host, ":", info$port, "/", info$dbname, "]")
 }
 #' @export
@@ -47,7 +50,7 @@ db_connection_describe.PostgreSQL <- db_connection_describe.PqConnection
 postgres_grepl <- function(pattern, x, ignore.case = FALSE, perl = FALSE, fixed = FALSE, useBytes = FALSE) {
   # https://www.postgresql.org/docs/current/static/functions-matching.html#FUNCTIONS-POSIX-TABLE
   if (any(c(perl, fixed, useBytes))) {
-    abort("`perl`, `fixed` and `useBytes` parameters are unsupported")
+    cli_abort("{.arg {c('perl', 'fixed', 'useBytes')}} parameters are unsupported.")
   }
 
   if (ignore.case) {
@@ -107,6 +110,33 @@ sql_translation.PqConnection <- function(con) {
       },
 
       # lubridate functions
+      # https://www.postgresql.org/docs/9.1/functions-datetime.html
+      day = function(x) {
+        sql_expr(EXTRACT(DAY %FROM% !!x))
+      },
+      mday = function(x) {
+        sql_expr(EXTRACT(DAY %FROM% !!x))
+      },
+      wday = function(x, label = FALSE, abbr = TRUE, week_start = NULL) {
+        if (!label) {
+          week_start <- week_start %||% getOption("lubridate.week.start", 7)
+          offset <- as.integer(7 - week_start)
+          sql_expr(EXTRACT("dow" %FROM% DATE(!!x) + !!offset) + 1)
+        } else if (label && !abbr) {
+          sql_expr(TO_CHAR(!!x, "Day"))
+        } else if (label && abbr) {
+          sql_expr(SUBSTR(TO_CHAR(!!x, "Day"), 1, 3))
+        } else {
+          cli_abort("Unrecognized arguments to {.arg wday}")
+        }
+      },
+      yday = function(x) sql_expr(EXTRACT(DOY %FROM% !!x)),
+      week = function(x) {
+        sql_expr(FLOOR ((EXTRACT(DOY %FROM% !!x) - 1L) / 7L) + 1L)
+      },
+      isoweek = function(x) {
+        sql_expr(EXTRACT(WEEK %FROM% !!x))
+      },
       month = function(x, label = FALSE, abbr = TRUE) {
         if (!label) {
           sql_expr(EXTRACT(MONTH %FROM% !!x))
@@ -120,7 +150,7 @@ sql_translation.PqConnection <- function(con) {
       },
       quarter = function(x, with_year = FALSE, fiscal_start = 1) {
         if (fiscal_start != 1) {
-          stop("`fiscal_start` is not supported in PostgreSQL translation. Must be 1.", call. = FALSE)
+          cli_abort("{.arg fiscal_start} is not supported in PostgreSQL translation. Must be 1.")
         }
 
         if (with_year) {
@@ -129,20 +159,9 @@ sql_translation.PqConnection <- function(con) {
           sql_expr(EXTRACT(QUARTER %FROM% !!x))
         }
       },
-      wday = function(x, label = FALSE, abbr = TRUE, week_start = NULL) {
-        if (!label) {
-          week_start <- week_start %||% getOption("lubridate.week.start", 7)
-          offset <- as.integer(7 - week_start)
-          sql_expr(EXTRACT("dow" %FROM% DATE(!!x) + !!offset) + 1)
-        } else if (label && !abbr) {
-          sql_expr(TO_CHAR(!!x, "Day"))
-        } else if (label && abbr) {
-          sql_expr(SUBSTR(TO_CHAR(!!x, "Day"), 1, 3))
-        } else {
-          stop("Unrecognized arguments to `wday`", call. = FALSE)
-        }
+      isoyear = function(x) {
+        sql_expr(EXTRACT(YEAR %FROM% !!x))
       },
-      yday = function(x) sql_expr(EXTRACT(DOY %FROM% !!x)),
 
       # https://www.postgresql.org/docs/13/datatype-datetime.html#DATATYPE-INTERVAL-INPUT
       seconds = function(x) {
@@ -234,4 +253,100 @@ sql_query_explain.PqConnection <- function(con, sql, format = "text", ...) {
 #' @export
 sql_query_explain.PostgreSQL <- sql_query_explain.PqConnection
 
-globalVariables(c("strpos", "%::%", "%FROM%", "DATE", "EXTRACT", "TO_CHAR", "string_agg", "%~*%", "%~%", "MONTH", "DOY", "DATE_TRUNC", "INTERVAL"))
+#' @export
+sql_query_insert.PqConnection <- function(con,
+                                          x_name,
+                                          y,
+                                          by,
+                                          conflict = c("error", "ignore"),
+                                          ...,
+                                          returning_cols = NULL,
+                                          method = NULL) {
+  method <- method %||% "on_conflict"
+  arg_match(method, c("on_conflict", "where_not_exists"), error_arg = "method")
+  if (method == "where_not_exists") {
+    return(NextMethod("sql_query_insert"))
+  }
+
+  # https://stackoverflow.com/questions/17267417/how-to-upsert-merge-insert-on-duplicate-update-in-postgresql
+  # https://www.sqlite.org/lang_UPSERT.html
+  conflict <- rows_check_conflict(conflict)
+
+  parts <- rows_insert_prep(con, x_name, y, by, lvl = 0)
+  by_sql <- escape(ident(by), parens = TRUE, collapse = ", ", con = con)
+
+  clauses <- list(
+    parts$insert_clause,
+    sql_clause_select(con, sql("*")),
+    sql_clause_from(parts$from),
+    sql_clause("ON CONFLICT", by_sql),
+    {if (conflict == "ignore") sql("DO NOTHING")},
+    sql_returning_cols(con, returning_cols, x_name)
+  )
+  sql_format_clauses(clauses, lvl = 0, con)
+}
+#' @export
+sql_query_insert.PostgreSQL <- sql_query_insert.PqConnection
+
+#' @export
+sql_query_upsert.PqConnection <- function(con,
+                                          x_name,
+                                          y,
+                                          by,
+                                          update_cols,
+                                          ...,
+                                          returning_cols = NULL,
+                                          method = NULL) {
+  method <- method %||% "on_conflict"
+  arg_match(method, c("cte_update", "on_conflict"), error_arg = "method")
+
+  if (method == "cte_update") {
+    return(NextMethod("sql_query_upsert"))
+  }
+
+  # https://stackoverflow.com/questions/17267417/how-to-upsert-merge-insert-on-duplicate-update-in-postgresql
+  # https://www.sqlite.org/lang_UPSERT.html
+  parts <- rows_prep(con, x_name, y, by, lvl = 0)
+
+  update_values <- set_names(
+    sql_table_prefix(con, update_cols, ident("excluded")),
+    update_cols
+  )
+  update_cols <- sql_escape_ident(con, update_cols)
+
+  insert_cols <- escape(ident(colnames(y)), collapse = ", ", parens = TRUE, con = con)
+  by_sql <- escape(ident(by), parens = TRUE, collapse = ", ", con = con)
+  clauses <- list(
+    sql_clause_insert(con, insert_cols, x_name),
+    sql_clause_select(con, sql("*")),
+    sql_clause_from(parts$from),
+    # `WHERE true` is required for SQLite
+    sql("WHERE true"),
+    sql_clause("ON CONFLICT ", by_sql),
+    sql("DO UPDATE"),
+    sql_clause_set(update_cols, update_values),
+    sql_returning_cols(con, returning_cols, x_name)
+  )
+  sql_format_clauses(clauses, lvl = 0, con)
+}
+
+#' @export
+sql_query_upsert.PostgreSQL <- sql_query_upsert.PqConnection
+
+#' @export
+sql_values_subquery.PqConnection <- sql_values_subquery_column_alias
+
+#' @export
+sql_values_subquery.PostgreSQL <- sql_values_subquery.PqConnection
+
+#' @export
+supports_window_clause.PqConnection <- function(con) {
+  TRUE
+}
+
+#' @export
+supports_window_clause.PostgreSQL <- function(con) {
+  TRUE
+}
+
+globalVariables(c("strpos", "%::%", "%FROM%", "DATE", "EXTRACT", "TO_CHAR", "string_agg", "%~*%", "%~%", "MONTH", "DOY", "DATE_TRUNC", "INTERVAL", "FLOOR", "WEEK"))
