@@ -1,13 +1,14 @@
 #' Backend: SQLite
 #'
 #' @description
-#' See `vignette("translate-function")` and `vignette("translate-verb")` for
+#' See `vignette("translation-function")` and `vignette("translation-verb")` for
 #' details of overall translation technology. Key differences for this backend
 #' are:
 #'
 #' * Uses non-standard `LOG()` function
 #' * Date-time extraction functions from lubridate
 #' * Custom median translation
+#' * Right and full joins are simulated using left joins
 #'
 #' Use `simulate_sqlite()` with `lazy_frame()` to see simulated SQL without
 #' converting to live access database.
@@ -42,15 +43,10 @@ sql_query_explain.SQLiteConnection <- function(con, sql, ...) {
 }
 
 #' @export
-sql_query_set_op.SQLiteConnection <- function(con, x, y, method, ..., all = FALSE) {
-  # SQLite does not allow parentheses
-  build_sql(
-    x,
-    "\n", sql(method), if (all) sql(" ALL"), "\n",
-    y,
-    con = con
-  )
-}
+sql_query_set_op.SQLiteConnection <- sql_query_set_op.Hive
+
+#' @export
+sql_query_upsert.SQLiteConnection <- sql_query_upsert.PqConnection
 
 sqlite_version <- function() {
   numeric_version(RSQLite::rsqliteVersion()[[2]])
@@ -104,7 +100,7 @@ sql_translation.SQLiteConnection <- function(con) {
         median = win_absent("median")
       )
     } else {
-      base_no_win
+      base_no_win # nocov
     }
   )
 }
@@ -117,14 +113,15 @@ sql_escape_logical.SQLiteConnection <- function(con, x){
 }
 
 #' @export
-sql_query_wrap.SQLiteConnection <- function(con, from, name = unique_subquery_name(), ...) {
+sql_query_wrap.SQLiteConnection <- function(con, from, name = NULL, ..., lvl = 0) {
   if (is.ident(from)) {
     setNames(from, name)
   } else {
+
     if (is.null(name)) {
-      build_sql("(", from, ")", con = con)
+      build_sql(sql_indent_subquery(from, con, lvl), con = con)
     } else {
-      build_sql("(", from, ") AS ", ident(name), con = con)
+      build_sql(sql_indent_subquery(from, con, lvl), " AS ", as_subquery_name(name), con = con)
     }
   }
 }
@@ -135,4 +132,62 @@ sql_expr_matches.SQLiteConnection <- function(con, x, y) {
   build_sql(x, " IS ", y, con = con)
 }
 
+#' @export
+sql_query_join.SQLiteConnection <- function(con, x, y, vars, type = "inner", by = NULL, na_matches = FALSE, ..., lvl = 0) {
+  # workaround as SQLite doesn't support FULL OUTER JOIN and RIGHT JOIN
+  # see: https://www.sqlite.org/omitted.html
+
+  if (type %in% c("left", "inner", "semi", "cross")) {
+    return(NextMethod())
+  }
+
+  # as `x` and `y` the vars also need to be swapped in vars and by
+  vars_right <- list(
+    alias = vars$alias,
+    x = vars$y,
+    y = vars$x,
+    all_x = vars$all_y,
+    all_y = vars$all_x
+  )
+  by_right <- list(
+    x = by$y,
+    y = by$x,
+    x_as = by$y_as,
+    y_as = by$x_as
+  )
+
+  if (type == "full") {
+    x_join <- sql_query_join(con, x, y, vars, type = "left", by = by, na_matches = na_matches, ..., lvl = lvl + 1)
+    y_join <- sql_query_join(con, y, x, vars_right, type = "left", by = by_right, na_matches = na_matches, ..., lvl = lvl + 1)
+    join_sql <- sql_query_set_op(
+      con,
+      x = x_join,
+      y = y_join,
+      method = "UNION",
+      lvl = lvl + 1
+    )
+
+    sql_query_select(
+      con,
+      select = ident(vars$alias),
+      from = dbplyr_sql_subquery(con, join_sql, lvl = lvl),
+      subquery = TRUE,
+      lvl = lvl
+    )
+  } else if (type == "right") {
+    sql_query_join(con, y, x, vars_right, type = "left", by = by_right, na_matches = na_matches, ..., lvl = lvl)
+  }
+}
+
+#' @export
+values_prepare.SQLiteConnection <- function(con, df) {
+  needs_escape <- purrr::map_lgl(df, ~ is(.x, "Date") || inherits(.x, "POSIXct"))
+  purrr::modify_if(df, needs_escape, ~ escape(.x, con = con, parens = FALSE, collapse = NULL))
+}
+
+#' @export
+supports_window_clause.SQLiteConnection <- function(con) {
+  TRUE
+}
+
 globalVariables(c("datetime", "NUMERIC", "REAL"))
