@@ -18,7 +18,7 @@
 #' @param from Either a string (giving a table name),
 #'   a fully qualified table name created by [in_schema()]
 #'   or a literal [sql()] string.
-#' @param ... Needed for compatibility with generic; currently ignored.
+#' @param ... Passed on to [tbl_sql()]
 #' @export
 #' @examples
 #' library(dplyr)
@@ -82,13 +82,24 @@
 #' @aliases tbl_dbi
 tbl.src_dbi <- function(src, from, ...) {
   subclass <- class(src$con)[[1]] # prefix added by dplyr::make_tbl
-  tbl_sql(c(subclass, "dbi"), src = src, from = from)
+  tbl_sql(c(subclass, "dbi"), src = src, from = from, ...)
 }
 
+# Internal calls to `tbl()` should be avoided in favor of tbl_src_dbi().
+# The former may query the database for column names if `vars` is omitted,
+# the latter always requires `vars`.
+tbl_src_dbi <- function(src, from, vars) {
+  force(vars)
+  tbl(src, from, vars = vars)
+}
+
+
+
 #' Database src
 #'
 #' @description
-#' \Sexpr[results=rd, stage=render]{lifecycle::badge("superseded")}
+#' `r lifecycle::badge("superseded")`
+#'
 #' Since can generate a `tbl()` directly from a DBI connection we no longer
 #' recommend using `src_dbi()`.
 #'
@@ -109,7 +120,7 @@ src_dbi <- function(con, auto_disconnect = FALSE) {
   if (is_false(auto_disconnect)) {
     disco <- NULL
   } else {
-    disco <- db_disconnector(con, quiet = is_true(auto_disconnect))
+    disco <- db_disconnector(con, quiet = is_true(auto_disconnect)) # nocov
   }
 
   subclass <- paste0("src_", class(con)[[1]])
@@ -125,7 +136,7 @@ src_dbi <- function(con, auto_disconnect = FALSE) {
 
 setOldClass(c("src_dbi", "src_sql", "src"))
 
-
+# nocov start
 # Creates an environment that disconnects the database when it's GC'd
 db_disconnector <- function(con, quiet = FALSE) {
   reg.finalizer(environment(), function(...) {
@@ -136,3 +147,4 @@ db_disconnector <- function(con, quiet = FALSE) {
   })
   environment()
 }
+# nocov end
