@@ -1,18 +1,27 @@
-context("test-tbl-lazy.R")
-
 test_that("adds src class", {
   tb <- tbl_lazy(mtcars, con = simulate_sqlite())
   expect_s3_class(tb, "tbl_SQLiteConnection")
 })
 
-test_that("has print method", {
-  expect_known_output(
-    tbl_lazy(mtcars),
-    test_path("test-tbl-lazy-print.txt"),
-    print = TRUE
+test_that("argument src is deprecated", {
+  expect_snapshot(dummy <- tbl_lazy(mtcars, src = simulate_sqlite()))
+})
+
+test_that("cannot convert tbl_lazy to data.frame", {
+  expect_snapshot(
+    error = TRUE,
+    as.data.frame(tbl_lazy(mtcars, src = simulate_sqlite()))
   )
 })
 
+test_that("dim() works for tbl_lazy", {
+  expect_equal(dim(tbl_lazy(mtcars)), c(NA, 11))
+})
+
+test_that("has print method", {
+  expect_snapshot(tbl_lazy(mtcars))
+})
+
 test_that("support colwise variants", {
   mf <- memdb_frame(x = 1:5, y = factor(letters[1:5]))
   exp <- mf %>% collect() %>% mutate(y = as.character(y))
