@@ -1,11 +1,9 @@
-context("test-verb-compute")
-
 test_that("collect equivalent to as.data.frame/as_tibble", {
   mf <- memdb_frame(letters)
 
   expect_equal(as.data.frame(mf), data.frame(letters, stringsAsFactors = FALSE))
-  expect_equal(tibble::as_tibble(mf), tibble::tibble(letters))
-  expect_equal(collect(mf), tibble::tibble(letters))
+  expect_equal(as_tibble(mf), tibble(letters))
+  expect_equal(collect(mf), tibble(letters))
 })
 
 test_that("explicit collection returns all data", {
@@ -13,7 +11,7 @@ test_that("explicit collection returns all data", {
   big <- memdb_frame(x = seq_len(n))
 
   nrow1 <- big %>% as.data.frame() %>% nrow()
-  nrow2 <- big %>% tibble::as_tibble() %>% nrow()
+  nrow2 <- big %>% as_tibble() %>% nrow()
   nrow3 <- big %>% collect() %>% nrow()
 
   expect_equal(nrow1, n)
@@ -65,6 +63,27 @@ test_that("compute creates correct column names", {
   expect_equal(out, tibble(x = 1, n = 1L))
 })
 
+test_that("compute keeps window and groups", {
+  out <- memdb_frame(x = 1, y = 1) %>%
+    window_order(x) %>%
+    group_by(x, y) %>%
+    summarise(n = n(), .groups = "drop_last") %>%
+    compute()
+
+  expect_equal(op_sort(out), list(quo(x)), ignore_formula_env = TRUE)
+  expect_equal(op_grps(out), "x")
+})
+
+test_that("compute can handle named name", {
+  name <- set_names(unique_subquery_name(), unique_subquery_name())
+  expect_equal(
+    memdb_frame(x = 1:10) %>%
+      compute() %>%
+      collect(),
+    tibble(x = 1:10)
+  )
+})
+
 # ops ---------------------------------------------------------------------
 
 test_that("sorting preserved across compute and collapse", {
