@@ -66,11 +66,16 @@ test_that("can rename with a function", {
 
 test_that("select preserves grouping vars", {
   mf <- memdb_frame(a = 1, b = 2) %>% group_by(b)
-  out <- mf %>% select(a) %>% collect()
+  expect_snapshot(out <- mf %>% select(a) %>% collect())
 
   expect_named(out, c("b", "a"))
 })
 
+test_that("select doesn't relocate grouping vars to the front", {
+  mf <- memdb_frame(a = 1, b = 2) %>% group_by(b)
+  expect_equal(mf %>% select(a, b) %>% op_vars(), c("a", "b"))
+})
+
 test_that("relocate works", {
   mf <- memdb_frame(a = 1, b = 2, c = 1) %>% group_by(b)
 
@@ -80,6 +85,26 @@ test_that("relocate works", {
   expect_named(out2, c("b", "c", "a"))
 })
 
+test_that("relocate can rename variables", {
+  mf <- memdb_frame(a = 1, b = 2, c = 1) %>% group_by(b)
+
+  out1 <- mf %>% relocate(d = b) %>% collect()
+  expect_named(out1, c("d", "a", "c"))
+  expect_equal(group_vars(out1), "d")
+})
+
+test_that("only add step if necessary", {
+  lf <- lazy_frame(x = 1:3, y = 1:3)
+  expect_equal(lf %>% select(everything()), lf)
+  expect_equal(lf %>% select(x, y), lf)
+
+  expect_equal(lf %>% rename(x = x), lf)
+  expect_equal(lf %>% rename(), lf)
+
+  expect_equal(lf %>% relocate(x, y), lf)
+  expect_equal(lf %>% relocate(), lf)
+})
+
 # sql_render --------------------------------------------------------------
 
 test_that("multiple selects are collapsed", {
@@ -97,6 +122,19 @@ test_that("mutate collapses over nested select", {
   expect_snapshot(lf %>% mutate(a = 1, b = 2) %>% select(x))
 })
 
+test_that("output is styled", {
+  local_reproducible_output(crayon = TRUE)
+
+  lf <- lazy_frame(x = 1, y = 1, z = 1)
+  out <- lf %>%
+    group_by(x) %>%
+    mutate(y = mean(y, na.rm = TRUE), z = z + 1) %>%
+    filter(z == 1) %>%
+    left_join(lf, by = "x")
+
+  expect_snapshot(show_query(out, cte = TRUE))
+})
+
 # sql_build -------------------------------------------------------------
 
 test_that("select picks variables", {
@@ -117,7 +155,7 @@ test_that("select renames variables", {
 
 test_that("select can refer to variables in local env", {
   vars <- c("x", "y")
-  out <- lazy_frame(x = 1, y = 1) %>%
+  out <- lazy_frame(x = 1, y = 1, z = 1) %>%
     select(dplyr::one_of(vars)) %>%
     sql_build()
 
@@ -155,3 +193,61 @@ test_that("mutate preserves grouping vars (#396)", {
   expect_equal(df %>% mutate(a = 1) %>% op_grps(), c("a", "b"))
   expect_equal(df %>% mutate(b = 1) %>% op_grps(), c("a", "b"))
 })
+
+
+# lazy_select_query -------------------------------------------------------
+
+test_that("select, relocate, and rename work", {
+  lf <- lazy_frame(x = 1, y = 1)
+
+  expect_equal(
+    lf %>%
+      select(x) %>%
+      .$lazy_query %>%
+      .$select,
+    new_lazy_select(exprs(x = x))
+  )
+
+  expect_equal(
+    lf %>%
+      relocate(y) %>%
+      .$lazy_query %>%
+      .$select,
+    new_lazy_select(exprs(y = y, x = x))
+  )
+
+  expect_equal(
+    lf %>%
+      rename(b = y, a = x) %>%
+      .$lazy_query %>%
+      .$select,
+    new_lazy_select(exprs(a = x, b = y))
+  )
+})
+
+test_that("renaming handles groups correctly", {
+  lf <- lazy_frame(x = 1, y = 1) %>%
+    group_by(x) %>%
+    rename(ax = x)
+
+  result <- lf$lazy_query
+  expect_equal(
+    result$select,
+    new_lazy_select(exprs(ax = x, y = y))
+  )
+
+  expect_equal(result$group_vars, "x")
+  expect_equal(op_grps(result), "ax")
+
+  result <- lf %>%
+    rename(x = ax) %>%
+    .$lazy_query
+
+  expect_equal(
+    result$select,
+    new_lazy_select(exprs(x = x, y = y))
+  )
+
+  expect_equal(result$group_vars, "x")
+  expect_equal(op_grps(result), "x")
+})
